@@ -9,6 +9,7 @@ import collections
 import contextlib
 import dis
 import io
+import locale
 import os
 import os.path
 import re
@@ -19,22 +20,20 @@ import warnings
 
 from pathlib import Path
 from typing import (
-    Any, Callable, Iterable, Iterator, List, NoReturn, Optional, Set, Tuple, Type,
-    TypeVar, Union, cast,
+    Any, Callable, NoReturn, TypeVar, cast,
 )
+from collections.abc import Iterable, Iterator
 
 import flaky
-import pytest
 
 from coverage import env
 from coverage.debug import DebugControl
 from coverage.exceptions import CoverageWarning
-from coverage.misc import output_encoding
-from coverage.types import TArc, TLineNo
+from coverage.types import TArc
 
 
-def run_command(cmd: str) -> Tuple[int, str]:
-    """Run a command in a sub-process.
+def run_command(cmd: str) -> tuple[int, str]:
+    """Run a command in a subprocess.
 
     Returns the exit status code and the combined stdout and stderr.
 
@@ -45,11 +44,13 @@ def run_command(cmd: str) -> Tuple[int, str]:
         with open("/tmp/processes.txt", "a") as proctxt:  # type: ignore[unreachable]
             print(os.getenv("PYTEST_CURRENT_TEST", "unknown"), file=proctxt, flush=True)
 
+    encoding = os.device_encoding(1) or locale.getpreferredencoding()
+
     # In some strange cases (PyPy3 in a virtualenv!?) the stdout encoding of
     # the subprocess is set incorrectly to ascii.  Use an environment variable
     # to force the encoding to be the same as ours.
     sub_env = dict(os.environ)
-    sub_env['PYTHONIOENCODING'] = output_encoding()
+    sub_env['PYTHONIOENCODING'] = encoding
 
     proc = subprocess.Popen(
         cmd,
@@ -63,7 +64,7 @@ def run_command(cmd: str) -> Tuple[int, str]:
     status = proc.returncode
 
     # Get the output, and canonicalize it to strings with newlines.
-    output_str = output.decode(output_encoding()).replace("\r", "")
+    output_str = output.decode(encoding).replace("\r", "")
     return status, output_str
 
 
@@ -74,7 +75,7 @@ def make_file(
     filename: str,
     text: str = "",
     bytes: bytes = b"",
-    newline: Optional[str] = None,
+    newline: str | None = None,
 ) -> str:
     """Create a file for testing.
 
@@ -115,8 +116,11 @@ def make_file(
             print(f"# {os.path.abspath(filename)}", file=fdis)
             cur_test = os.getenv("PYTEST_CURRENT_TEST", "unknown")
             print(f"# PYTEST_CURRENT_TEST = {cur_test}", file=fdis)
+            kwargs = {}
+            if env.PYVERSION >= (3, 13):
+                kwargs["show_offsets"] = True
             try:
-                dis.dis(text, file=fdis)
+                dis.dis(text, file=fdis, **kwargs)
             except Exception as exc:
                 # Some tests make .py files that aren't Python, so dis will
                 # fail, which is expected.
@@ -147,7 +151,7 @@ class CheckUniqueFilenames:
     """Asserts the uniqueness of file names passed to a function."""
 
     def __init__(self, wrapped: Callable[..., Any]) -> None:
-        self.filenames: Set[str] = set()
+        self.filenames: set[str] = set()
         self.wrapped = wrapped
 
     @classmethod
@@ -175,7 +179,7 @@ class CheckUniqueFilenames:
         return self.wrapped(filename, *args, **kwargs)
 
 
-def re_lines(pat: str, text: str, match: bool = True) -> List[str]:
+def re_lines(pat: str, text: str, match: bool = True) -> list[str]:
     """Return a list of lines selected by `pat` in the string `text`.
 
     If `match` is false, the selection is inverted: only the non-matching
@@ -219,7 +223,7 @@ _arcz_map.update({c: ord(c) - ord('0') for c in '123456789'})
 _arcz_map.update({c: 10 + ord(c) - ord('A') for c in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'})
 
 
-def arcz_to_arcs(arcz: str) -> List[TArc]:
+def arcz_to_arcs(arcz: str) -> list[TArc]:
     """Convert a compact textual representation of arcs to a list of pairs.
 
     The text has space-separated pairs of letters.  Period is -1, 1-9 are
@@ -255,40 +259,8 @@ def arcz_to_arcs(arcz: str) -> List[TArc]:
     return sorted(arcs)
 
 
-_arcz_unmap = {val: ch for ch, val in _arcz_map.items()}
-
-
-def _arcs_to_arcz_repr_one(num: TLineNo) -> str:
-    """Return an arcz form of the number `num`, or "?" if there is none."""
-    if num == -1:
-        return "."
-    z = ""
-    if num < 0:
-        z += "-"
-        num *= -1
-    z += _arcz_unmap.get(num, "?")
-    return z
-
-
-def arcs_to_arcz_repr(arcs: Optional[Iterable[TArc]]) -> str:
-    """Convert a list of arcs to a readable multi-line form for asserting.
-
-    Each pair is on its own line, with a comment showing the arcz form,
-    to make it easier to decode when debugging test failures.
-
-    """
-    repr_list = []
-    for a, b in (arcs or ()):
-        line = repr((a, b))
-        line += " # "
-        line += _arcs_to_arcz_repr_one(a)
-        line += _arcs_to_arcz_repr_one(b)
-        repr_list.append(line)
-    return "\n".join(repr_list) + "\n"
-
-
 @contextlib.contextmanager
-def change_dir(new_dir: Union[str, Path]) -> Iterator[None]:
+def change_dir(new_dir: str | Path) -> Iterator[None]:
     """Change directory, and then change back.
 
     Use as a context manager, it will return to the original
@@ -305,8 +277,8 @@ def change_dir(new_dir: Union[str, Path]) -> Iterator[None]:
 T = TypeVar("T")
 
 def assert_count_equal(
-    a: Optional[Iterable[T]],
-    b: Optional[Iterable[T]],
+    a: Iterable[T] | None,
+    b: Iterable[T] | None,
 ) -> None:
     """
     A pytest-friendly implementation of assertCountEqual.
@@ -321,7 +293,7 @@ def assert_count_equal(
 
 def assert_coverage_warnings(
     warns: Iterable[warnings.WarningMessage],
-    *msgs: Union[str, re.Pattern[str]],
+    *msgs: str | re.Pattern[str],
 ) -> None:
     """
     Assert that the CoverageWarning's in `warns` have `msgs` as messages.
@@ -343,7 +315,7 @@ def assert_coverage_warnings(
 @contextlib.contextmanager
 def swallow_warnings(
     message: str = r".",
-    category: Type[Warning] = CoverageWarning,
+    category: type[Warning] = CoverageWarning,
 ) -> Iterator[None]:
     """Swallow particular warnings.
 
@@ -354,15 +326,9 @@ def swallow_warnings(
         yield
 
 
-xfail_pypy38 = pytest.mark.xfail(
-    env.PYPY and env.PYVERSION[:2] == (3, 8) and env.PYPYVERSION < (7, 3, 11),
-    reason="These tests fail on older PyPy 3.8",
-)
-
-
 class FailingProxy:
     """A proxy for another object, but one method will fail a few times before working."""
-    def __init__(self, obj: Any, methname: str, fails: List[Exception]) -> None:
+    def __init__(self, obj: Any, methname: str, fails: list[Exception]) -> None:
         """Create the failing proxy.
 
         `obj` is the object to proxy.  `methname` is the method that will fail
